@@ -1,5 +1,26 @@
+/*
+    Copyright (C) 2002-2005, Jason Katz-Brown <jasonkb@mit.edu>
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+*/
+
+#include "kolf.h"
+
 #include <kaction.h>
-#include <kapplication.h>
+#include <kactioncollection.h>
+#include <ktoggleaction.h>
 #include <kconfig.h>
 #include <kdebug.h>
 #include <kdeversion.h>
@@ -7,33 +28,24 @@
 #include <kglobal.h>
 #include <kio/netaccess.h>
 #include <klocale.h>
-#include <kmainwindow.h>
+#include <kxmlguiwindow.h>
 #include <kmimetype.h>
 #include <kmessagebox.h>
-#include <kprinter.h>
 #include <kscoredialog.h>
 #include <kstandarddirs.h>
 #include <kstatusbar.h>
-#include <kstdaccel.h>
-#include <kstdaction.h>
-#include <kstdgameaction.h>
-#include <kstdguiitem.h>
+#include <kstandardshortcut.h>
+#include <kstandardaction.h>
+#include <kstandardgameaction.h>
+#include <KStandardGuiItem>
+#include <kicon.h>
+#include <kselectaction.h>
+#include <kdeprintdialog.h>
 
-#include <qcolor.h>
-#include <qevent.h>
-#include <qfile.h>
-#include <qobject.h>
-#include <qmap.h>
-#include <qpoint.h>
-#include <qtimer.h>
-#include <qptrlist.h>
-#include <qpixmap.h>
-#include <qpixmapcache.h>
-#include <qfileinfo.h>
-#include <qstring.h>
-#include <qstringlist.h>
-#include <qlayout.h>
-#include <qwidget.h>
+#include <QTimer>
+#include <QGridLayout>
+#include <QtGui/QPrinter>
+#include <QtGui/QPrintDialog>
 
 #include <stdlib.h>
 
@@ -45,11 +57,11 @@
 #include "editor.h"
 #include "pluginloader.h"
 #include "printdialogpage.h"
-#include "kolf.h"
 
 Kolf::Kolf()
-    : KMainWindow(0, "Kolf")
+    : KXmlGuiWindow(0)
 {
+	setObjectName("Kolf");
 	competition = false;
 	game = 0;
 	editor = 0;
@@ -57,15 +69,15 @@ Kolf::Kolf()
 	scoreboard = 0;
 	isTutorial = false;
 
-	initGUI();
+	setupActions();
 
 	obj = new ObjectList;
 	initPlugins();
 
-	filename = QString::null;
+	filename = QString();
 	dummy = new QWidget(this);
 	setCentralWidget(dummy);
-	layout = new QGridLayout(dummy, 3, 1);
+	layout = new QGridLayout(dummy);
 
 	resize(420, 480);
 }
@@ -73,83 +85,139 @@ Kolf::Kolf()
 Kolf::~Kolf()
 {
 	// wipe out our objects
-	obj->setAutoDelete(true);
+	while (!obj->isEmpty())
+		delete obj->takeFirst();
 	delete obj;
 }
 
-void Kolf::initGUI()
+void Kolf::setupActions()
 {
-	newAction = KStdGameAction::gameNew(this, SLOT(newGame()), actionCollection());
-	newAction->setText(newAction->text() + QString("..."));
+	// Game
+	newAction = KStandardGameAction::gameNew(this, SLOT(newGame()), actionCollection());
+	endAction = KStandardGameAction::end(this, SLOT(closeGame()), actionCollection());
+	//printAction = KStandardGameAction::print(this, SLOT(print()), actionCollection());
+	KStandardGameAction::quit(this, SLOT(close()), actionCollection());
 
-	endAction = KStdGameAction::end(this, SLOT(closeGame()), actionCollection());
-	printAction = KStdGameAction::print(this, SLOT(print()), actionCollection());
-
-	(void) KStdGameAction::quit(this, SLOT(close()), actionCollection());
-	saveAction = KStdAction::save(this, SLOT(save()), actionCollection(), "game_save");
+	saveAction = actionCollection()->addAction(KStandardAction::Save, "game_save", this, SLOT(save()));
 	saveAction->setText(i18n("Save &Course"));
-	saveAsAction = KStdAction::saveAs(this, SLOT(saveAs()), actionCollection(), "game_save_as");
+	saveAsAction = actionCollection()->addAction(KStandardAction::SaveAs, "game_save_as", this, SLOT(saveAs()));
 	saveAsAction->setText(i18n("Save &Course As..."));
 
-	saveGameAction = new KAction(i18n("&Save Game"), 0, this, SLOT(saveGame()), actionCollection(), "savegame");
-	saveGameAsAction = new KAction(i18n("&Save Game As..."), 0, this, SLOT(saveGameAs()), actionCollection(), "savegameas");
+	saveGameAction = actionCollection()->addAction("savegame");
+	saveGameAction->setText(i18n("&Save Game"));
+	connect(saveGameAction, SIGNAL(triggered(bool) ), SLOT(saveGame()));
+	saveGameAsAction = actionCollection()->addAction("savegameas");
+	saveGameAsAction->setText(i18n("&Save Game As..."));
+	connect(saveGameAsAction, SIGNAL(triggered(bool) ), SLOT(saveGameAs()));
 
-	loadGameAction = KStdGameAction::load(this, SLOT(loadGame()), actionCollection());
-	loadGameAction->setText(i18n("Load Saved Game..."));
+	loadGameAction = KStandardGameAction::load(this, SLOT(loadGame()), actionCollection());
+	highScoreAction = KStandardGameAction::highscores(this, SLOT(showHighScores()), actionCollection());
 
-	highScoreAction = KStdGameAction::highscores(this, SLOT(showHighScores()), actionCollection());
-
-	editingAction = new KToggleAction(i18n("&Edit"), "pencil", CTRL+Key_E, this, SLOT(emptySlot()), actionCollection(), "editing");
-	newHoleAction = new KAction(i18n("&New"), "filenew", CTRL+SHIFT+Key_N, this, SLOT(emptySlot()), actionCollection(), "newhole");
-	clearHoleAction = new KAction(KStdGuiItem::clear().text(), "locationbar_erase", CTRL+Key_Delete, this, SLOT(emptySlot()), actionCollection(), "clearhole");
-	resetHoleAction = new KAction(i18n("&Reset"), CTRL+Key_R, this, SLOT(emptySlot()), actionCollection(), "resethole");
-	undoShotAction = KStdAction::undo(this, SLOT(emptySlot()), actionCollection(), "undoshot");
+	// Hole
+	editingAction = new KToggleAction(KIcon("document-properties"), i18n("&Edit"), this);
+	actionCollection()->addAction("editing", editingAction);
+	connect(editingAction, SIGNAL(triggered(bool) ), SLOT(emptySlot()));
+	editingAction->setShortcut(Qt::CTRL+Qt::Key_E);
+	newHoleAction = actionCollection()->addAction("newhole");
+	newHoleAction->setIcon(KIcon("document-new"));
+	newHoleAction->setText(i18n("&New"));
+	connect(newHoleAction, SIGNAL(triggered(bool)), SLOT(emptySlot()));
+	newHoleAction->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_N);
+	clearHoleAction = actionCollection()->addAction("clearhole");
+	clearHoleAction->setIcon(KIcon("edit-clear-locationbar-ltr"));
+	clearHoleAction->setText(KStandardGuiItem::clear().text());
+	connect(clearHoleAction, SIGNAL(triggered(bool)), SLOT(emptySlot()));
+	clearHoleAction->setShortcut(Qt::CTRL+Qt::Key_Delete);
+	resetHoleAction = actionCollection()->addAction("resethole");
+	resetHoleAction->setText(i18n("&Reset"));
+	connect(resetHoleAction, SIGNAL(triggered(bool) ), SLOT(emptySlot()));
+	resetHoleAction->setShortcut(Qt::CTRL+Qt::Key_R);
+	undoShotAction = KStandardAction::undo(this, SLOT(emptySlot()), this);
+	actionCollection()->addAction("undoshot", undoShotAction);
 	undoShotAction->setText(i18n("&Undo Shot"));
 	//replayShotAction = new KAction(i18n("&Replay Shot"), 0, this, SLOT(emptySlot()), actionCollection(), "replay");
 
-	holeAction = new KListAction(i18n("Switch to Hole"), 0, this, SLOT(emptySlot()), actionCollection(), "switchhole");
-	nextAction = new KAction(i18n("&Next Hole"), "forward", KStdAccel::shortcut(KStdAccel::Forward), this, SLOT(emptySlot()), actionCollection(), "nexthole");
-	prevAction = new KAction(i18n("&Previous Hole"), "back", KStdAccel::shortcut(KStdAccel::Back), this, SLOT(emptySlot()), actionCollection(), "prevhole");
-	firstAction = new KAction(i18n("&First Hole"), "gohome", KStdAccel::shortcut(KStdAccel::Home), this, SLOT(emptySlot()), actionCollection(), "firsthole");
-	lastAction = new KAction(i18n("&Last Hole"), CTRL+SHIFT+Key_End, this, SLOT(emptySlot()), actionCollection(), "lasthole");
-	randAction = new KAction(i18n("&Random Hole"), "goto", 0, this, SLOT(emptySlot()), actionCollection(), "randhole");
+	// Go
+	holeAction = new KSelectAction(i18n("Switch to Hole"), this);
+	actionCollection()->addAction("switchhole", holeAction);
+	connect(holeAction, SIGNAL(triggered(bool)), SLOT(emptySlot()));
+	nextAction = actionCollection()->addAction("nexthole");
+	nextAction->setIcon(KIcon("go-next"));
+	nextAction->setText(i18n("&Next Hole"));
+	connect(nextAction, SIGNAL(triggered(bool)), SLOT(emptySlot()));
+	nextAction->setShortcuts(KStandardShortcut::shortcut(KStandardShortcut::Forward));
+	prevAction = actionCollection()->addAction("prevhole");
+	prevAction->setIcon(KIcon("go-previous"));
+	prevAction->setText(i18n("&Previous Hole"));
+	connect(prevAction, SIGNAL(triggered(bool)), SLOT(emptySlot()));
+	prevAction->setShortcuts(KStandardShortcut::shortcut(KStandardShortcut::Back));
+	firstAction = actionCollection()->addAction("firsthole");
+	firstAction->setIcon(KIcon("go-home"));
+	firstAction->setText(i18n("&First Hole"));
+	connect(firstAction, SIGNAL(triggered(bool)), SLOT(emptySlot()));
+	firstAction->setShortcuts(KStandardShortcut::shortcut(KStandardShortcut::Begin));
+	lastAction = actionCollection()->addAction("lasthole");
+	lastAction->setText(i18n("&Last Hole"));
+	connect(lastAction, SIGNAL(triggered(bool) ), SLOT(emptySlot()));
+	lastAction->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_End); // why not KStandardShortcut::End (Ctrl+End)?
+	randAction = actionCollection()->addAction("randhole");
+	randAction->setIcon(KIcon("go-jump"));
+	randAction->setText(i18n("&Random Hole"));
+	connect(randAction, SIGNAL(triggered(bool)), SLOT(emptySlot()));
 
-	useMouseAction = new KToggleAction(i18n("Enable &Mouse for Moving Putter"), 0, this, SLOT(emptySlot()), actionCollection(), "usemouse");
-	useMouseAction->setCheckedState(i18n("Disable &Mouse for Moving Putter"));
+	// Settings
+	useMouseAction = new KToggleAction(i18n("Enable &Mouse for Moving Putter"), this);
+	actionCollection()->addAction("usemouse", useMouseAction);
+	connect(useMouseAction, SIGNAL(triggered(bool) ), SLOT(emptySlot()));
 	connect(useMouseAction, SIGNAL(toggled(bool)), this, SLOT(useMouseChanged(bool)));
-	KConfig *config = kapp->config();
-	config->setGroup("Settings");
-	useMouseAction->setChecked(config->readBoolEntry("useMouse", true));
+	KConfigGroup configGroup(KGlobal::config(), "Settings");
+	useMouseAction->setChecked(configGroup.readEntry("useMouse", true));
 
-	useAdvancedPuttingAction = new KToggleAction(i18n("Enable &Advanced Putting"), 0, this, SLOT(emptySlot()), actionCollection(), "useadvancedputting");
-	useAdvancedPuttingAction->setCheckedState(i18n("Disable &Advanced Putting"));
+	useAdvancedPuttingAction = new KToggleAction(i18n("Enable &Advanced Putting"), this);
+	actionCollection()->addAction("useadvancedputting", useAdvancedPuttingAction);
+	connect(useAdvancedPuttingAction, SIGNAL(triggered(bool) ), SLOT(emptySlot()));
 	connect(useAdvancedPuttingAction, SIGNAL(toggled(bool)), this, SLOT(useAdvancedPuttingChanged(bool)));
-	useAdvancedPuttingAction->setChecked(config->readBoolEntry("useAdvancedPutting", false));
+	useAdvancedPuttingAction->setChecked(configGroup.readEntry("useAdvancedPutting", false));
 
-	showInfoAction = new KToggleAction(i18n("Show &Info"), "info", CTRL+Key_I, this, SLOT(emptySlot()), actionCollection(), "showinfo");
-	showInfoAction->setCheckedState(i18n("Hide &Info"));
+	showInfoAction = new KToggleAction(KIcon("help-about"), i18n("Show &Info"), this);
+	actionCollection()->addAction("showinfo", showInfoAction);
+	connect(showInfoAction, SIGNAL(triggered(bool) ), SLOT(emptySlot()));
+	showInfoAction->setShortcut(Qt::CTRL+Qt::Key_I);
 	connect(showInfoAction, SIGNAL(toggled(bool)), this, SLOT(showInfoChanged(bool)));
-	showInfoAction->setChecked(config->readBoolEntry("showInfo", false));
+	showInfoAction->setChecked(configGroup.readEntry("showInfo", true));
 
-	showGuideLineAction = new KToggleAction(i18n("Show Putter &Guideline"), 0, this, SLOT(emptySlot()), actionCollection(), "showguideline");
-	showGuideLineAction->setCheckedState(i18n("Hide Putter &Guideline"));
+	showGuideLineAction = new KToggleAction(i18n("Show Putter &Guideline"), this);
+	actionCollection()->addAction("showguideline", showGuideLineAction);
+	connect(showGuideLineAction, SIGNAL(triggered(bool) ), SLOT(emptySlot()));
 	connect(showGuideLineAction, SIGNAL(toggled(bool)), this, SLOT(showGuideLineChanged(bool)));
-	showGuideLineAction->setChecked(config->readBoolEntry("showGuideLine", true));
+	showGuideLineAction->setChecked(configGroup.readEntry("showGuideLine", true));
 
-	KToggleAction *act=new KToggleAction(i18n("Enable All Dialog Boxes"), 0, this, SLOT(enableAllMessages()), actionCollection(), "enableAll");
-	act->setCheckedState(i18n("Disable All Dialog Boxes"));
+	KToggleAction *act = new KToggleAction(i18n("Enable All Dialog Boxes"), this);
+	actionCollection()->addAction("enableAll", act);
+	connect(act, SIGNAL(triggered(bool) ), SLOT(enableAllMessages()));
 
-	soundAction = new KToggleAction(i18n("Play &Sounds"), 0, this, SLOT(emptySlot()), actionCollection(), "sound");
+	soundAction = new KToggleAction(i18n("Play &Sounds"), this);
+	actionCollection()->addAction("sound", soundAction);
+	connect(soundAction, SIGNAL(triggered(bool) ), SLOT(emptySlot()));
 	connect(soundAction, SIGNAL(toggled(bool)), this, SLOT(soundChanged(bool)));
-	soundAction->setChecked(config->readBoolEntry("sound", true));
+	soundAction->setChecked(configGroup.readEntry("sound", true));
 
-	(void) new KAction(i18n("&Reload Plugins"), 0, this, SLOT(initPlugins()), actionCollection(), "reloadplugins");
-	(void) new KAction(i18n("Show &Plugins"), 0, this, SLOT(showPlugins()), actionCollection(), "showplugins");
+	//There are no external plugins in 4.0 and no ported instructions on how to write them
+	//Plugins will still be loaded at startup IF one is made, but reduce interface options for now
+	//QAction *action = actionCollection()->addAction("reloadplugins");
+	//action->setText(i18n("&Reload Plugins"));
+	//connect(action, SIGNAL(triggered(bool) ), SLOT(initPlugins()));
+	//action = actionCollection()->addAction("showplugins");
+	//action->setText(i18n("Show &Plugins"));
+	//connect(action, SIGNAL(triggered(bool) ), SLOT(showPlugins()));
 
-	aboutAction = new KAction(i18n("&About Course"), 0, this, SLOT(emptySlot()), actionCollection(), "aboutcourse");
-	tutorialAction = new KAction(i18n("&Tutorial"), 0, this, SLOT(tutorial()), actionCollection(), "tutorial");
+	aboutAction = actionCollection()->addAction("aboutcourse");
+	aboutAction->setText(i18n("&About Course"));
+	connect(aboutAction, SIGNAL(triggered(bool) ), SLOT(emptySlot()));
+	tutorialAction = actionCollection()->addAction("tutorial");
+	tutorialAction->setText(i18n("&Tutorial"));
+	connect(tutorialAction, SIGNAL(triggered(bool) ), SLOT(tutorial()));
 
-	statusBar();
 	setupGUI();
 }
 
@@ -168,14 +236,14 @@ void Kolf::startNewGame()
 
 	if (loadedGame.isNull())
 	{
-		dialog = new NewGameDialog(filename.isNull(), dummy, "New Game Dialog");
+		dialog = new NewGameDialog(filename.isNull());
 		if (dialog->exec() != QDialog::Accepted)
 			goto end;
 	}
 
 	players.clear();
 	delete scoreboard;
-	scoreboard = new ScoreBoard(dummy, "Score Board");
+	scoreboard = new ScoreBoard(dummy);
 	layout->addWidget(scoreboard, 1, 0);
 	scoreboard->show();
 
@@ -183,11 +251,11 @@ void Kolf::startNewGame()
 	{
 		PlayerEditor *curEditor = 0;
 		int newId = 1;
-		for (curEditor = dialog->players()->first(); curEditor; curEditor = dialog->players()->next(), ++newId)
+		for (curEditor = dialog->players()->at(newId-1); newId <= dialog->players()->count(); ++newId)
 		{
 			players.append(Player());
-			players.last().ball()->setColor(curEditor->color());
-			players.last().setName(curEditor->name());
+			players.last().ball()->setColor(dialog->players()->at(newId-1)->color());
+			players.last().setName(dialog->players()->at(newId-1)->name());
 			players.last().setId(newId);
 		}
 
@@ -197,18 +265,18 @@ void Kolf::startNewGame()
 	else
 	{
 		KConfig config(loadedGame);
-		config.setGroup("0 Saved Game");
+		KConfigGroup configGroup(config.group("0 Saved Game"));
 
 		if (isTutorial)
 			filename = KGlobal::dirs()->findResource("appdata", "tutorial.kolf");
 		else
-			filename = config.readEntry("Course", QString::null);
+			filename = configGroup.readEntry("Course", QString());
 
 		if (filename.isNull())
 			return;
 
-		competition = config.readBoolEntry("Competition", false);
-		firstHole = config.readNumEntry("Current Hole", 1);
+		competition = configGroup.readEntry("Competition", false);
+		firstHole = configGroup.readEntry("Current Hole", 1);
 
 		players.clear();
 		KolfGame::scoresFromSaved(&config, players);
@@ -239,19 +307,19 @@ void Kolf::startNewGame()
 	connect(game, SIGNAL(titleChanged(const QString &)), this, SLOT(titleChanged(const QString &)));
 	connect(game, SIGNAL(newStatusText(const QString &)), this, SLOT(newStatusText(const QString &)));
 	connect(game, SIGNAL(currentHole(int)), this, SLOT(setCurrentHole(int)));
-	connect(holeAction, SIGNAL(activated(const QString &)), game, SLOT(switchHole(const QString &)));
-	connect(nextAction, SIGNAL(activated()), game, SLOT(nextHole()));
-	connect(prevAction, SIGNAL(activated()), game, SLOT(prevHole()));
-	connect(firstAction, SIGNAL(activated()), game, SLOT(firstHole()));
-	connect(lastAction, SIGNAL(activated()), game, SLOT(lastHole()));
-	connect(randAction, SIGNAL(activated()), game, SLOT(randHole()));
-	connect(editingAction, SIGNAL(activated()), game, SLOT(toggleEditMode()));
-	connect(newHoleAction, SIGNAL(activated()), game, SLOT(addNewHole()));
-	connect(clearHoleAction, SIGNAL(activated()), game, SLOT(clearHole()));
-	connect(resetHoleAction, SIGNAL(activated()), game, SLOT(resetHole()));
-	connect(undoShotAction, SIGNAL(activated()), game, SLOT(undoShot()));
-	//connect(replayShotAction, SIGNAL(activated()), game, SLOT(replay()));
-	connect(aboutAction, SIGNAL(activated()), game, SLOT(showInfoDlg()));
+	connect(holeAction, SIGNAL(triggered(const QString &)), game, SLOT(switchHole(const QString &)));
+	connect(nextAction, SIGNAL(triggered(bool)), game, SLOT(nextHole()));
+	connect(prevAction, SIGNAL(triggered(bool)), game, SLOT(prevHole()));
+	connect(firstAction, SIGNAL(triggered(bool)), game, SLOT(firstHole()));
+	connect(lastAction, SIGNAL(triggered(bool)), game, SLOT(lastHole()));
+	connect(randAction, SIGNAL(triggered(bool)), game, SLOT(randHole()));
+	connect(editingAction, SIGNAL(triggered(bool)), game, SLOT(toggleEditMode()));
+	connect(newHoleAction, SIGNAL(triggered(bool)), game, SLOT(addNewHole()));
+	connect(clearHoleAction, SIGNAL(triggered(bool)), game, SLOT(clearHole()));
+	connect(resetHoleAction, SIGNAL(triggered(bool)), game, SLOT(resetHole()));
+	connect(undoShotAction, SIGNAL(triggered(bool)), game, SLOT(undoShot()));
+	//connect(replayShotAction, SIGNAL(triggered(bool)), game, SLOT(replay()));
+	connect(aboutAction, SIGNAL(triggered(bool)), game, SLOT(showInfoDlg()));
 	connect(useMouseAction, SIGNAL(toggled(bool)), game, SLOT(setUseMouse(bool)));
 	connect(useAdvancedPuttingAction, SIGNAL(toggled(bool)), game, SLOT(setUseAdvancedPutting(bool)));
 	connect(soundAction, SIGNAL(toggled(bool)), game, SLOT(setSound(bool)));
@@ -264,7 +332,7 @@ void Kolf::startNewGame()
 	game->setShowGuideLine(showGuideLineAction->isChecked());
 	game->setSound(soundAction->isChecked());
 
-	layout->addWidget(game, 0, 0, AlignCenter);
+	layout->addWidget(game, 0, 0);//, Qt::AlignCenter);
 
 	game->show();
 	game->setFocus();
@@ -275,7 +343,7 @@ void Kolf::startNewGame()
 	setHoleOtherEnabled(true);
 	aboutAction->setEnabled(true);
 	highScoreAction->setEnabled(true);
-	printAction->setEnabled(true);
+	//printAction->setEnabled(true);
 	saveAction->setEnabled(true);
 	saveAsAction->setEnabled(true);
 	saveGameAction->setEnabled(true);
@@ -287,7 +355,6 @@ void Kolf::startNewGame()
 	loadGameAction->setEnabled(false);
 	tutorialAction->setEnabled(false);
 
-
 	// so game can do stuff that needs to be done
 	// after things above are connected
 	game->startFirstHole(firstHole);
@@ -299,7 +366,7 @@ void Kolf::startNewGame()
 void Kolf::newGame()
 {
 	isTutorial = false;
-	filename = QString::null;
+	filename = QString();
 	startNewGame();
 }
 
@@ -309,13 +376,13 @@ void Kolf::tutorial()
 	if (newfilename.isNull())
 	        return;
 
-	filename = QString::null;
+	filename = QString();
 	loadedGame = newfilename;
 	isTutorial = true;
 
 	startNewGame();
 
-	loadedGame = QString::null;
+	loadedGame = QString();
 }
 
 void Kolf::closeGame()
@@ -327,19 +394,19 @@ void Kolf::closeGame()
 		game->pause();
 	}
 
-	filename = QString::null;
+	filename = QString();
 
 	editingEnded();
 	delete game;
 	game = 0;
-	loadedGame = QString::null;
+	loadedGame = QString();
 
 	editingAction->setChecked(false);
 	setEditingEnabled(false);
 	endAction->setEnabled(false);
 	aboutAction->setEnabled(false);
 	highScoreAction->setEnabled(false);
-	printAction->setEnabled(false);
+	//printAction->setEnabled(false);
 	saveAction->setEnabled(false);
 	saveAsAction->setEnabled(false);
 	saveGameAction->setEnabled(false);
@@ -353,7 +420,7 @@ void Kolf::closeGame()
 	loadGameAction->setEnabled(true);
 	tutorialAction->setEnabled(true);
 
-	titleChanged(QString::null);
+	titleChanged(QString());
 	updateModified(false);
 
 	QTimer::singleShot(100, this, SLOT(createSpacer()));
@@ -364,19 +431,19 @@ void Kolf::createSpacer()
 	// make a player to play the spacer hole
 	spacerPlayers.clear();
 	spacerPlayers.append(Player());
-	spacerPlayers.last().ball()->setColor(yellow);
+	spacerPlayers.last().ball()->setColor(Qt::yellow);
 	spacerPlayers.last().setName("player");
 	spacerPlayers.last().setId(1);
 
 	delete spacer;
 	spacer = new KolfGame(obj, &spacerPlayers, KGlobal::dirs()->findResource("appdata", "intro"), dummy);
 	spacer->setSound(false);
-	spacer->startFirstHole(1);
-	layout->addWidget(spacer, 0, 0, AlignCenter);
-	spacer->hidePutter();
+	layout->addWidget(spacer, 0, 0);//, Qt::AlignCenter);
 	spacer->ignoreEvents(true);
 
 	spacer->show();
+	spacer->startFirstHole(1);
+	spacer->hidePutter();
 }
 
 void Kolf::gameOver()
@@ -403,6 +470,7 @@ void Kolf::gameOver()
 		if (curName == i18n("Par"))
 		{
 			curPar = curScore;
+			curScore = 0;
 			continue;
 		}
 
@@ -431,10 +499,10 @@ void Kolf::gameOver()
 		if (names.count() > 1)
 		{
 			QString winners = names.join(i18n(" and "));
-			KMessageBox::information(this, i18n("%1 tied").arg(winners));
+			KMessageBox::information(this, i18n("%1 tied", winners));
 		}
 		else
-			KMessageBox::information(this, i18n("%1 won!").arg(names.first()));
+			KMessageBox::information(this, i18n("%1 won!", names.first()));
 	}
 
 	if (competition)
@@ -453,13 +521,14 @@ void Kolf::gameOver()
 		for (HighScoreList::Iterator it = highScores.begin(); it != highScores.end(); ++it)
 		{
 			KScoreDialog::FieldInfo info;
-			info[KScoreDialog::Name] = (*it).name;
+                        info[KScoreDialog::Name] = (*it).name;
+                        info[KScoreDialog::Score].setNum((*it).score);
 			info[KScoreDialog::Custom1] = QString::number(curPar);
 
-			scoreDialog->addScore((*it).score, info, false, true);
+                        scoreDialog->addScore(info, KScoreDialog::LessIsMore);
 		}
 
-		scoreDialog->setComment(i18n("High Scores for %1").arg(courseInfo.name));
+		scoreDialog->setComment(i18n("High Scores for %1", courseInfo.name));
 		scoreDialog->show();
 	}
 
@@ -475,7 +544,7 @@ void Kolf::showHighScores()
 	game->courseInfo(courseInfo, game->curFilename());
 
 	scoreDialog->setConfigGroup(courseInfo.untranslatedName + QString(" Highscores"));
-	scoreDialog->setComment(i18n("High Scores for %1").arg(courseInfo.name));
+	scoreDialog->setComment(i18n("High Scores for %1", courseInfo.name));
 	scoreDialog->show();
 }
 
@@ -487,15 +556,16 @@ void Kolf::save()
 		return;
 	}
 
-	if (game)
+	if (game) {
 		game->save();
-
-	game->setFocus();
+		game->setFocus();
+	}
 }
 
 void Kolf::saveAs()
 {
-	QString newfilename = KFileDialog::getSaveFileName(":kourses", "application/x-kourse", this, i18n("Pick Kolf Course to Save To"));
+	QString newfilename = KFileDialog::getSaveFileName( KUrl("kfiledialog:///kourses"),
+                                  "application/x-kourse", this, i18n("Pick Kolf Course to Save To"));
 	if (!newfilename.isNull())
 	{
 		filename = newfilename;
@@ -507,7 +577,8 @@ void Kolf::saveAs()
 
 void Kolf::saveGameAs()
 {
-	QString newfilename = KFileDialog::getSaveFileName(":savedkolf", "application/x-kolf", this, i18n("Pick Saved Game to Save To"));
+	QString newfilename = KFileDialog::getSaveFileName( KUrl("kfiledialog:///savedkolf"),
+                                    "application/x-kolf", this, i18n("Pick Saved Game to Save To"));
 	if (newfilename.isNull())
 		return;
 
@@ -525,19 +596,20 @@ void Kolf::saveGame()
 	}
 
 	KConfig config(loadedGame);
-	config.setGroup("0 Saved Game");
+	KConfigGroup configGroup(config.group("0 Saved Game"));
 
-	config.writeEntry("Competition", competition);
-	config.writeEntry("Course", filename);
+	configGroup.writeEntry("Competition", competition);
+	configGroup.writeEntry("Course", filename);
 
 	game->saveScores(&config);
 
-	config.sync();
+	configGroup.sync();
 }
 
 void Kolf::loadGame()
 {
-	loadedGame = KFileDialog::getOpenFileName(":savedkolf", QString::fromLatin1("application/x-kolf"), this, i18n("Pick Kolf Saved Game"));
+	loadedGame = KFileDialog::getOpenFileName( KUrl("kfiledialog:///savedkolf"),
+			 QLatin1String("application/x-kolf"), this, i18n("Pick Kolf Saved Game"));
 
 	if (loadedGame.isNull())
 		return;
@@ -546,8 +618,8 @@ void Kolf::loadGame()
 	startNewGame();
 }
 
-// called by main for commmand line files
-void Kolf::openURL(KURL url)
+// called by main for command line files
+void Kolf::openUrl(KUrl url)
 {
 	QString target;
 	if (KIO::NetAccess::download(url, target, this))
@@ -572,12 +644,12 @@ void Kolf::openURL(KURL url)
 
 void Kolf::newPlayersTurn(Player *player)
 {
-	tempStatusBarText = i18n("%1's turn").arg(player->name());
+	tempStatusBarText = i18n("%1's turn", player->name());
 
 	if (showInfoAction->isChecked())
-		statusBar()->message(tempStatusBarText, 5 * 1000);
+		statusBar()->showMessage(tempStatusBarText, 5 * 1000);
 	else
-		statusBar()->message(tempStatusBarText);
+		statusBar()->showMessage(tempStatusBarText);
 
 	scoreboard->setCurrentCell(player->id() - 1, game->currentHole() - 1);
 }
@@ -585,15 +657,16 @@ void Kolf::newPlayersTurn(Player *player)
 void Kolf::newStatusText(const QString &text)
 {
 	if (text.isEmpty())
-		statusBar()->message(tempStatusBarText);
+		statusBar()->showMessage(tempStatusBarText);
 	else
-		statusBar()->message(text);
+		statusBar()->showMessage(text);
 }
 
 void Kolf::editingStarted()
 {
 	delete editor;
-	editor = new Editor(obj, dummy, "Editor");
+	editor = new Editor(obj, dummy);
+	editor->setObjectName( "Editor" );
 	connect(editor, SIGNAL(addNewItem(Object *)), game, SLOT(addNewObject(Object *)));
 	connect(editor, SIGNAL(changed()), game, SLOT(setModified()));
 	connect(editor, SIGNAL(addNewItem(Object *)), this, SLOT(setHoleFocus()));
@@ -643,7 +716,7 @@ void Kolf::inPlayEnd()
 
 void Kolf::maxStrokesReached(const QString &name)
 {
-	KMessageBox::sorry(this, i18n("%1's score has reached the maximum for this hole.").arg(name));
+	KMessageBox::sorry(this, i18n("%1's score has reached the maximum for this hole.", name));
 }
 
 void Kolf::updateHoleMenu(int largest)
@@ -694,15 +767,21 @@ void Kolf::checkEditing()
 
 void Kolf::print()
 {
-	KPrinter pr;
-	pr.addDialogPage(new PrintDialogPage());
+	if (!game)
+		return;
 
-    if (pr.setup(this, i18n("Print %1 - Hole %2").arg(game->courseName()).arg(game->currentHole())))
+	QPrinter pr;
+	PrintDialogPage prPage;
+
+	QPrintDialog *printDialog = KdePrint::createPrintDialog(&pr, QList<QWidget*>() << &prPage, this);
+	printDialog->setWindowTitle(i18n("Print %1 - Hole %2", game->courseName(), game->currentHole()));
+
+	if (printDialog->exec())
 	{
 		pr.newPage();
-		if (game)
-			game->print(pr);
+		game->print(pr, prPage.printTitle());
 	}
+	delete printDialog;
 }
 
 void Kolf::updateModified(bool mod)
@@ -719,38 +798,37 @@ void Kolf::titleChanged(const QString &newTitle)
 
 void Kolf::useMouseChanged(bool yes)
 {
-	KConfig *config = kapp->config(); config->setGroup("Settings"); config->writeEntry("useMouse", yes); config->sync();
+	KConfigGroup configGroup(KGlobal::config(), "Settings"); configGroup.writeEntry("useMouse", yes); configGroup.sync();
 }
 
 void Kolf::useAdvancedPuttingChanged(bool yes)
 {
-	KConfig *config = kapp->config(); config->setGroup("Settings"); config->writeEntry("useAdvancedPutting", yes); config->sync();
+	KConfigGroup configGroup(KGlobal::config(), "Settings"); configGroup.writeEntry("useAdvancedPutting", yes); configGroup.sync();
 }
 
 void Kolf::showInfoChanged(bool yes)
 {
-	KConfig *config = kapp->config(); config->setGroup("Settings"); config->writeEntry("showInfo", yes); config->sync();
+	KConfigGroup configGroup(KGlobal::config(), "Settings"); configGroup.writeEntry("showInfo", yes); configGroup.sync();
 }
 
 void Kolf::showGuideLineChanged(bool yes)
 {
-	KConfig *config = kapp->config(); config->setGroup("Settings"); config->writeEntry("showGuideLine", yes); config->sync();
+	KConfigGroup configGroup(KGlobal::config(), "Settings"); configGroup.writeEntry("showGuideLine", yes); configGroup.sync();
 }
 
 void Kolf::soundChanged(bool yes)
 {
-	KConfig *config = kapp->config(); config->setGroup("Settings"); config->writeEntry("sound", yes); config->sync();
+	KConfigGroup configGroup(KGlobal::config(), "Settings"); configGroup.writeEntry("sound", yes); configGroup.sync();
 }
 
 void Kolf::initPlugins()
 {
-	//kdDebug(12007) << "initPlugins" << endl;
+	//kDebug(12007) << "initPlugins";
 	if (game)
 		game->pause();
 
-	obj->setAutoDelete(true);
-	obj->clear();
-	plugins.setAutoDelete(false);
+	while (!obj->isEmpty())
+		delete obj->takeFirst();
 	plugins.clear();
 
 	// add prefab objects
@@ -767,11 +845,11 @@ void Kolf::initPlugins()
 	obj->append(new BumperObj());
 
 	ObjectList *other = PluginLoader::loadAll();
-	Object *object = 0;
-	for (object = other->first(); object; object = other->next())
+	QList<Object *>::const_iterator object;
+	for (object = other->constBegin(); object != other->constEnd(); ++object)
 	{
-		obj->append(object);
-		plugins.append(object);
+		obj->append(*object);
+		plugins.append(*object);
 	}
 
 	if (game)
@@ -780,21 +858,30 @@ void Kolf::initPlugins()
 		game->unPause();
 	}
 
-	//kdDebug(12007) << "end of initPlugins" << endl;
+	//kDebug(12007) << "end of initPlugins";
 }
 
 void Kolf::showPlugins()
 {
 	QString text = QString("<h2>%1</h2><ol>").arg(i18n("Currently Loaded Plugins"));
-	Object *object = 0;
+	QList<Object *>::const_iterator object;
+        for (object = plugins.constBegin(); object != plugins.constEnd(); ++object)
+	{
+		text.append("<li>");
+		text.append((*object)->name());
+		text.append(" - ");
+		text.append(i18n("by %1", (*object)->author()));
+		text.append("</li>");
+	}
+	/*Object *object = 0;
 	for (object = plugins.first(); object; object = plugins.next())
 	{
 		text.append("<li>");
 		text.append(object->name());
 		text.append(" - ");
-		text.append(i18n("by %1").arg(object->author()));
+		text.append(i18n("by %1", object->author()));
 		text.append("</li>");
-	}
+	}*/
 	text.append("</ol>");
 	KMessageBox::information(this, text, i18n("Plugins"));
 }
@@ -806,7 +893,7 @@ void Kolf::enableAllMessages()
 
 void Kolf::setCurrentHole(int hole)
 {
-	if (!holeAction)
+	if (!holeAction || holeAction->items().count() < hole)
 		return;
 	// Golf is 1-based, KListAction is 0-based
 	holeAction->setCurrentItem(hole - 1);
